﻿using DressAtelierContracts.BindingModels;
using DressAtelierContracts.SearchModels;
using DressAtelierContracts.StorageContracts;
using DressAtelierContracts.ViewModels;
using DressAtelierDatabaseImplementation.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace DressAtelierDatabaseImplementation.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new DressAtelierDatabase();
            return context.Orders.Include(x => x.Dress).Select(x => x.GetViewModel).ToList(); 
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new DressAtelierDatabase();
            return context.Orders.Include(x => x.Dress).Where(x => x.ID == model.ID).ToList().Select(x => x.GetViewModel).ToList();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.ID.HasValue)
            {
                return null;
            }

            using var context = new DressAtelierDatabase();
            return context.Orders.Include(x => x.Dress).FirstOrDefault(x => x.ID == model.ID)?.GetViewModel;
            
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new DressAtelierDatabase();
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders.Include(x => x.Dress).FirstOrDefault(x => x.ID == model.ID)?.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new DressAtelierDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var order = context.Orders.FirstOrDefault(x => x.ID == model.ID);
                if(order == null)
                {
                    return null;
                }
                order.Update(model);
                context.SaveChanges();
                return context.Orders.Include(x => x.Dress).FirstOrDefault(x => x.ID == model.ID)?.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new DressAtelierDatabase();
            var order = context.Orders.FirstOrDefault(x => x.ID == model.ID);
            if(order == null)
            {
                return null;
            }
            context.Orders.Remove(order);
            context.SaveChanges();           
            return context.Orders.Include(x => x.Dress).FirstOrDefault(x => x.ID == model.ID)?.GetViewModel;
        }

    }
}
